@@ -2,8 +2,9 @@
 .. include:: ../README.md
 """
 __docformat__ = "restructuredtext"
+__version__ = "0.13.0"
 
-__version__ = "0.9.1"
+import sys
 
 # pylint: disable=cyclic-import
 from .command import CmdContext, Command, Options  # noqa: F401
@@ -13,6 +14,9 @@ from .redirect import Redirect
 from .result import PipeResult, Result, ResultError  # noqa: F401
 from .runner import AUDIT_EVENT_SUBPROCESS_SPAWN  # noqa: F401
 
+if sys.platform != "win32":
+    from .watcher import DefaultChildWatcher  # noqa: F401
+
 STDOUT = Redirect.STDOUT
 DEVNULL = Redirect.DEVNULL
 CAPTURE = Redirect.CAPTURE
