@@ -40,9 +40,6 @@ _Audit_Fn_T = Optional[Callable[[str, dict], None]]  # pylint: disable=invalid-n
 class Options:  # pylint: disable=too-many-instance-attributes
     "Concrete class for per-command options."
 
-    context: "CmdContext" = field(compare=False, repr=False)
-    "Root context object."
-
     env: Optional[ImmutableDict] = field(default=None, repr=False)
     "Additional environment variables for command."
 
@@ -103,7 +100,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     pass_fds_close: bool = False
     "True if pass_fds should be closed after subprocess launch."
 
-    write_mode: bool = False
+    writable: bool = False
     "True if using process substitution in write mode."
 
     _start_new_session: bool = False
@@ -187,14 +184,9 @@ class Options:  # pylint: disable=too-many-instance-attributes
 class CmdContext:
     """Concrete class for an immutable execution context."""
 
-    options: Options = None  # type: ignore
+    options: Options = Options()
     "Default command options."
 
-    def __post_init__(self):
-        if self.options is None:
-            # Initialize `context` in Options to `self`.
-            object.__setattr__(self, "options", Options(self))
-
     def stdin(self, input_, *, close=False) -> "CmdContext":
         "Return new context with updated `input` settings."
         new_options = self.options.set_stdin(input_, close)
@@ -229,7 +221,7 @@ class CmdContext:
         alt_name=_UNSET,
         pass_fds=_UNSET,
         pass_fds_closed=_UNSET,
-        write_mode=_UNSET,
+        writable=_UNSET,
         _start_new_session=_UNSET,
         _preexec_fn=_UNSET,
         pty=_UNSET,
@@ -246,48 +238,7 @@ class CmdContext:
 
     def __call__(self, *args):
         "Construct a new command."
-        return Command(self._coerce(args), self.options)
-
-    def _cmd_apply(self, cmd, args):
-        """Apply arguments to an existing command.
-
-        This method is an extension point.
-        """
-        return Command(
-            cmd.args + self._coerce(args),
-            cmd.options,
-        )
-
-    def _pipe_apply(self, _pipe, args):  # pylint: disable=no-self-use
-        """Apply arguments to an existing pipeline.
-
-        This method is an extension point.
-        """
-        assert len(args) > 0
-        raise TypeError("Calling pipeline with 1 or more arguments.")
-
-    def _coerce(self, args):
-        """Flatten lists and coerce arguments to string.
-
-        This method is an extension point.
-        """
-        result = []
-        for arg in args:
-            if isinstance(
-                arg, (str, bytes, bytearray, os.PathLike, Command, shellous.Pipeline)
-            ):
-                result.append(arg)
-            elif isinstance(arg, (list, tuple)):
-                result.extend(self._coerce(arg))
-            elif isinstance(arg, (dict, set)):
-                raise NotImplementedError("syntax is reserved")
-            elif arg is Ellipsis:
-                raise NotImplementedError("syntax is reserved")
-            elif arg is None:
-                raise TypeError("None in argument list")
-            else:
-                result.append(str(arg))
-        return tuple(result)
+        return Command(coerce(args), self.options)
 
 
 @dataclass(frozen=True)
@@ -371,7 +322,7 @@ class Command:
         alt_name: Unset[Optional[str]] = _UNSET,
         pass_fds: Unset[Iterable[int]] = _UNSET,
         pass_fds_close: Unset[bool] = _UNSET,
-        write_mode: Unset[bool] = _UNSET,
+        writable: Unset[bool] = _UNSET,
         _start_new_session: Unset[bool] = _UNSET,
         _preexec_fn: Unset[_Preexec_Fn_T] = _UNSET,
         pty: Unset[bool] = _UNSET,
@@ -437,7 +388,7 @@ class Command:
         Close the file descriptors in `pass_fds` immediately in the current
         process immediately after launching the subprocess.
 
-        **write_mode** (bool) default=False<br>
+        **writable** (bool) default=False<br>
         Used to indicate process substitution is writing.
 
         **_start_new_session** (bool) default=False<br>
@@ -547,7 +498,7 @@ class Command:
         "Apply more arguments to the end of the command."
         if not args:
             return self
-        return self.options.context._cmd_apply(self, args)
+        return Command(self.args + coerce(args), self.options)
 
     def __str__(self):
         """Return string representation for command.
@@ -575,11 +526,44 @@ class Command:
             return self.stdout(rhs, append=True)
         return NotImplemented
 
-    def __invert__(self):
-        "Unary ~ operator sets write_mode to True."
-        return self.set(write_mode=True)
+    def __mod__(self, rhs):
+        "Modulo operator is used to concatenate commands."
+        if isinstance(rhs, Command):
+            return self(rhs.args)
+        return NotImplemented
+
+    @property
+    def writable(self):
+        "Set `writable` to True."
+        return self.set(writable=True)
+
+    @property
+    def result(self):
+        "Set `return_result` and `exit_codes`."
+        return self.set(return_result=True, exit_codes=range(-255, 256))
 
 
 def _check_args(out, append):
     if append and not isinstance(out, STDOUT_APPEND_TYPES):
         raise TypeError(f"{type(out)} does not support append")
+
+
+def coerce(args):
+    """Flatten lists and coerce arguments to string."""
+    result = []
+    for arg in args:
+        if isinstance(
+            arg, (str, bytes, bytearray, os.PathLike, Command, shellous.Pipeline)
+        ):
+            result.append(arg)
+        elif isinstance(arg, (list, tuple)):
+            result.extend(coerce(arg))
+        elif isinstance(arg, (dict, set)):
+            raise NotImplementedError("syntax is reserved")
+        elif arg is Ellipsis:
+            raise NotImplementedError("syntax is reserved")
+        elif arg is None:
+            raise TypeError("None in argument list")
+        else:
+            result.append(str(arg))
+    return tuple(result)
