@@ -13,10 +13,11 @@ from shellous.harvest import harvest, harvest_results
 from shellous.log import LOG_DETAIL, LOG_ENTER, LOG_EXIT, LOGGER, log_method, log_timer
 from shellous.redirect import Redirect
 from shellous.result import Result, make_result
-from shellous.util import close_fds, uninterrupted, verify_dev_fd, wait_pid, which
+from shellous.util import close_fds, poll_wait_pid, uninterrupted, verify_dev_fd, which
 
 _KILL_TIMEOUT = 3.0
 _CLOSE_TIMEOUT = 0.25
+_UNKNOWN_EXIT_CODE = 255
 _UNLAUNCHED_EXIT_CODE = -255
 
 _BSD = sys.platform.startswith("freebsd") or sys.platform == "darwin"
@@ -32,21 +33,22 @@ def _is_cmd(cmd):
     return isinstance(cmd, (shellous.Command, shellous.Pipeline))
 
 
-def _is_write_mode(cmd):
-    "Return true if command/pipeline has write_mode set."
+def _is_writable(cmd):
+    "Return true if command/pipeline has `writable` set."
     if isinstance(cmd, shellous.Pipeline):
         # Pipelines need to check both the last/first commands.
-        return cmd.options.write_mode or cmd[0].options.write_mode
-    return cmd.options.write_mode
+        return cmd.options.writable or cmd[0].options.writable
+    return cmd.options.writable
 
 
-def _split(encoding):
+def _enc(encoding):
+    "Helper function to split the encoding name into codec and errors."
     if encoding is None:
         raise TypeError("when encoding is None, input must be bytes")
     return encoding.split(maxsplit=1)
 
 
-class _RunOptions:  # pylint: disable=too-many-instance-attributes
+class _RunOptions:
     """_RunOptions is context manager to assist in running a command.
 
     This class sets up low-level I/O redirection and helps close open file
@@ -81,7 +83,7 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
             return self
         except Exception as ex:
             if LOG_DETAIL:
-                LOGGER.warning("_RunOptions.enter %r ex=%r", self.command.name, ex)
+                LOGGER.debug("_RunOptions.enter %r ex=%r", self.command.name, ex)
             _cleanup(self.command)
             raise
 
@@ -90,7 +92,7 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
         self.close_fds()
         if exc_value:
             if LOG_DETAIL:
-                LOGGER.warning(
+                LOGGER.debug(
                     "_RunOptions.exit %r exc_value=%r",
                     self.command.name,
                     exc_value,
@@ -115,7 +117,7 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
                 continue
 
             (read_fd, write_fd) = os.pipe()
-            if _is_write_mode(arg):
+            if _is_writable(arg):
                 new_args.append(f"/dev/fd/{write_fd}")
                 pass_fds.append(write_fd)
                 subcmd = arg.stdin(read_fd, close=True)
@@ -173,6 +175,8 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
             )
             start_session = True
 
+        assert not preexec_fn or callable(preexec_fn)
+
         self.input_bytes = input_bytes
         self.kwd_args = {
             "stdin": stdin,
@@ -219,7 +223,7 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
             if close:
                 self.open_fds.append(stdin)
         elif isinstance(input_, str):
-            input_bytes = input_.encode(*_split(encoding))
+            input_bytes = input_.encode(*_enc(encoding))
         elif isinstance(input_, (asyncio.StreamReader, io.BytesIO, io.StringIO)):
             # Shellous-supported input classes.
             assert stdin == asyncio.subprocess.PIPE
@@ -293,7 +297,8 @@ class _RunOptions:  # pylint: disable=too-many-instance-attributes
         if stderr == asyncio.subprocess.STDOUT:
             stderr = child_fd
 
-        return stdin, stdout, stderr, lambda: pty_util.set_ctty(child_fd)
+        ttyname = os.ttyname(child_fd)
+        return stdin, stdout, stderr, lambda: pty_util.set_ctty(ttyname)
 
 
 class Runner:
@@ -322,6 +327,7 @@ class Runner:
         self._tasks = []
         self._timer = None  # asyncio.TimerHandle
         self._timed_out = False  # True if runner timeout expired
+        self._last_signal = None
 
     @property
     def name(self):
@@ -344,8 +350,16 @@ class Runner:
     def returncode(self):
         "Process's exit code."
         if not self._proc:
+            if self._cancelled:
+                # The process was cancelled before starting.
+                return _UNLAUNCHED_EXIT_CODE
             return None
-        return self._proc.returncode
+        code = self._proc.returncode
+        if code == _UNKNOWN_EXIT_CODE and self._last_signal is not None:
+            # Rarely after sending a SIGTERM, waitpid fails to locate the child
+            # process. In this case, map the status to the last signal we sent.
+            return -self._last_signal  # pylint: disable=invalid-unary-operand-type
+        return code
 
     @property
     def cancelled(self):
@@ -354,12 +368,8 @@ class Runner:
 
     def result(self, output_bytes=b""):
         "Check process exit code and raise a ResultError if necessary."
-        if self._proc:
-            code = self._proc.returncode
-        else:
-            # The process was cancelled before starting.
-            assert self._cancelled
-            code = _UNLAUNCHED_EXIT_CODE
+        code = self.returncode
+        assert code is not None
 
         result = Result(
             output_bytes,
@@ -398,23 +408,19 @@ class Runner:
             self._tasks.clear()  # all tasks were cancelled
             await self._kill()
 
+        except Exception as ex:
+            LOGGER.info("Runner.wait exited with error %r ex=%r", self, ex)
+            self._tasks.clear()  # all tasks were cancelled
+            await self._kill()
+            raise  # re-raise exception
+
     @log_method(LOG_DETAIL)
     async def _wait_pid(self):
         "Manually poll `waitpid` until process finishes."
         assert self._is_bsd_pty()
         while True:
-            status = wait_pid(self._proc.pid)
-            if status is not None:
-                LOGGER.debug(
-                    "process %r exited with returncode %r (wait_pid)",
-                    self._proc.pid,
-                    status,
-                )
-                # pylint: disable=protected-access
-                self._proc._transport._returncode = status
-                self._proc._transport._proc.returncode = status
+            if poll_wait_pid(self._proc):
                 break
-
             await asyncio.sleep(0.025)
 
     @log_method(LOG_DETAIL)
@@ -442,7 +448,7 @@ class Runner:
                 self._set_cancelled()
             await self._kill_wait()
 
-        except Exception as ex:
+        except (Exception, GeneratorExit) as ex:
             LOGGER.warning("Runner.kill %r ex=%r", self, ex)
             await self._kill_wait()
             raise
@@ -457,6 +463,7 @@ class Runner:
             self._proc.kill()
         else:
             self._proc.send_signal(sig)
+            self._last_signal = sig
 
     @log_method(LOG_DETAIL)
     async def _kill_wait(self):
@@ -471,8 +478,12 @@ class Runner:
             self._send_signal(None)
             await harvest(self._waiter(), timeout=_KILL_TIMEOUT, trustee=self)
         except asyncio.TimeoutError as ex:
-            LOGGER.error("%r failed to kill process %r", self, self._proc)
-            raise RuntimeError(f"Unable to kill process {self._proc!r}") from ex
+            # Manually check if the process is still running.
+            if poll_wait_pid(self._proc):
+                LOGGER.warning("%r process reaped manually %r", self, self._proc)
+            else:
+                LOGGER.error("%r failed to kill process %r", self, self._proc)
+                raise RuntimeError(f"Unable to kill process {self._proc!r}") from ex
 
     @log_method(LOG_ENTER, _info=True)
     async def __aenter__(self):
@@ -636,7 +647,7 @@ class Runner:
             return None
 
         if isinstance(source, io.StringIO):
-            input_bytes = source.getvalue().encode(*_split(opts.encoding))
+            input_bytes = source.getvalue().encode(*_enc(opts.encoding))
             self.add_task(redir.write_stream(input_bytes, stream, eof), tag)
             return None
 
@@ -753,7 +764,7 @@ class Runner:
         "Return string representation of Runner."
         cancelled = " cancelled" if self._cancelled else ""
         if self._proc:
-            procinfo = f" pid={self._proc.pid} exit_code={self._proc.returncode}"
+            procinfo = f" pid={self._proc.pid} exit_code={self.returncode}"
         else:
             procinfo = " pid=None"
         return f"<Runner {self.name!r}{cancelled}{procinfo}>"
@@ -796,7 +807,7 @@ class Runner:
         return run.result(bytes(output_bytes))
 
 
-class PipeRunner:  # pylint: disable=too-many-instance-attributes
+class PipeRunner:
     """PipeRunner is an asynchronous context manager that runs a pipeline.
 
     ```
@@ -848,7 +859,6 @@ class PipeRunner:  # pylint: disable=too-many-instance-attributes
     async def _wait(self, *, kill=False):
         "Wait for pipeline to finish."
         assert self._results is None
-        assert self._tasks is not None
 
         if kill:
             LOGGER.info("PipeRunner.wait killing pipe %r", self)
@@ -858,6 +868,7 @@ class PipeRunner:  # pylint: disable=too-many-instance-attributes
         cancelled, self._results = await harvest_results(*self._tasks, trustee=self)
         if cancelled:
             self._cancelled = True
+        self._tasks.clear()  # clear all tasks when done
 
     @log_method(LOG_ENTER)
     async def __aenter__(self):
@@ -886,6 +897,7 @@ class PipeRunner:  # pylint: disable=too-many-instance-attributes
     async def _finish(self, exc_value):
         "Wait for pipeline to exit and handle cancellation."
         if exc_value is not None:
+            LOGGER.warning("PipeRunner._finish exc_value=%r", exc_value)
             if _is_cancelled(exc_value):
                 self._cancelled = True
             await self._wait(kill=True)
@@ -955,10 +967,11 @@ class PipeRunner:  # pylint: disable=too-many-instance-attributes
         last_coro = cmds[-1].coro(_run_future=last_fut)
         middle_coros = [cmd.coro() for cmd in cmds[1:-1]]
 
-        self.add_task(first_coro)
-        for coro in middle_coros:
-            self.add_task(coro)
-        self.add_task(last_coro)
+        # Tag each task name with the index of the command in the pipe.
+        self.add_task(first_coro, "0")
+        for i, coro in enumerate(middle_coros):
+            self.add_task(coro, str(i + 1))
+        self.add_task(last_coro, str(len(cmds) - 1))
 
         # When capturing, we need the first and last commands in the
         # pipe to signal when they are ready.
